from xml.dom.minidom import Document
from niprov.format import Format
## Decided not to go with prov python lib as it depends on lxml which depends on c binaries


class XmlFormat(Format):

    def serializeSingle(self, item):
        return self.serializeList([item])

    def serializeList(self, itemOrList):
        prov = 'http://www.w3.org/ns/prov#'
        nfo = 'http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#'
        dct = 'http://purl.org/dc/terms/'
        dom = Document()
        doc = dom.createElementNS(prov, 'prov:document')
        doc.setAttribute('xmlns:prov', prov)
        doc.setAttribute('xmlns:nfo', nfo)
        doc.setAttribute('xmlns:dct', dct)
        dom.appendChild(doc)
        for i, item in enumerate(itemOrList):
            entity = dom.createElementNS(prov, 'prov:entity')
            entityId = 'niprov:file'+str(i)
            entity.setAttribute('id', entityId)

            fileUrl = dom.createElementNS(nfo, 'nfo:fileUrl')
            fileUrlVal = dom.createTextNode(item.location.toUrl())
            fileUrl.appendChild(fileUrlVal)
            entity.appendChild(fileUrl)

            if 'size' in item.provenance:

                fileSize = dom.createElementNS(nfo, 'nfo:fileSize')
                fileSizeVal = dom.createTextNode(str(item.provenance['size']))
                fileSize.appendChild(fileSizeVal)
                entity.appendChild(fileSize)

            if 'created' in item.provenance:

                fileLastMod = dom.createElementNS(nfo, 'nfo:fileLastModified')
                fileLastModVal = dom.createTextNode(item.provenance['created'].isoformat())
                fileLastMod.appendChild(fileLastModVal)
                entity.appendChild(fileLastMod)

            if 'hash' in item.provenance:

                fileHash = dom.createElementNS(nfo, 'nfo:FileHash')
                hashId = entityId+'.hash'
                fileHash.setAttribute('id', hashId)

                hashAlgo = dom.createElementNS(nfo, 'nfo:hashAlgorithm')
                hashAlgoVal = dom.createTextNode('MD5')
                hashAlgo.appendChild(hashAlgoVal)
                fileHash.appendChild(hashAlgo)

                hashValue = dom.createElementNS(nfo, 'nfo:hashValue')
                hashValueVal = dom.createTextNode(item.provenance['hash'])
                hashValue.appendChild(hashValueVal)
                fileHash.appendChild(hashValue)

                hasHash = dom.createElementNS(nfo, 'nfo:hasHash')
                hasHashVal = dom.createTextNode(hashId)
                hasHash.appendChild(hasHashVal)
                entity.appendChild(hasHash)

                doc.appendChild(fileHash)

            if 'transformation' in item.provenance:

                act = dom.createElementNS(prov, 'prov:activity')
                activityId = entityId+'.xform'
                act.setAttribute('id', activityId)

                actTitle = dom.createElementNS(dct, 'dct:title')
                actTitleVal = dom.createTextNode(item.provenance['transformation'])
                actTitle.appendChild(actTitleVal)
                act.appendChild(actTitle)
                doc.appendChild(act)

                wasGen = dom.createElementNS(prov, 'prov:wasGeneratedBy')
                entityRef = dom.createElementNS(prov, 'prov:entity')
                entityRef.setAttribute('prov:ref', entityId)
                wasGen.appendChild(entityRef)
                activityRef = dom.createElementNS(prov, 'prov:activity')
                activityRef.setAttribute('prov:ref', activityId)
                wasGen.appendChild(activityRef)
                doc.appendChild(wasGen)

                if 'created' in item.provenance:
                    wasGenTime = dom.createElementNS(prov, 'prov:time')
                    wasGenTimeVal = dom.createTextNode(item.provenance['created'].isoformat())
                    wasGenTime.appendChild(wasGenTimeVal)
                    wasGen.appendChild(wasGenTime)

            doc.appendChild(entity)

        return dom.toprettyxml(encoding="UTF-8")





from niprov.dependencies import Dependencies
from niprov.formatjson import JsonFormat
from niprov.formatxml import XmlFormat
from niprov.formatnarrated import NarratedFormat
from niprov.formatsimple import SimpleFormat
from niprov.formatdict import DictFormat
from niprov.formatobject import ObjectFormat


class FormatFactory(object):

    def __init__(self, dependencies=Dependencies()):
        self.dependencies = dependencies

    def create(self, formatName):
        if formatName == 'json':
            return JsonFormat(self.dependencies)
        if formatName == 'xml':
            return XmlFormat(self.dependencies)
        if formatName == 'narrated':
            return NarratedFormat()
        if formatName == 'simple':
            return SimpleFormat()
        if formatName == 'dict':
            return DictFormat()
        if formatName == 'object':
            return ObjectFormat()
        raise ValueError('Unknown format: '+str(formatName))



#!/usr/bin/python
# -*- coding: UTF-8 -*-


class DirectMedium(object):
    """Dummy  Medium class which simply returns the serialized provenance 
    passed.
    """

    def export(self, formattedProvenance):
        return formattedProvenance




from mock import Mock, patch, call
from tests.ditest import DependencyInjectionTestBase


class MediumFactoryTests(DependencyInjectionTestBase):

    def setUp(self):
        super(MediumFactoryTests, self).setUp()

    def test_Provides_stdout(self):
        from niprov.mediumfactory import MediumFactory
        from niprov.mediumstdout import StandardOutputMedium
        factory = MediumFactory()
        self.assertIsInstance(factory.create('stdout'), StandardOutputMedium)

    def test_Provides_direct(self):
        from niprov.mediumfactory import MediumFactory
        from niprov.mediumdirect import DirectMedium
        factory = MediumFactory()
        self.assertIsInstance(factory.create('direct'), DirectMedium)

    def test_Provides_file(self):
        from niprov.mediumfactory import MediumFactory
        from niprov.mediumfile import FileMedium
        factory = MediumFactory()
        self.assertIsInstance(factory.create('file'), FileMedium)

    def test_Raises_exception_on_unknown_name(self):
        from niprov.mediumfactory import MediumFactory
        factory = MediumFactory()
        with self.assertRaisesRegexp(ValueError, 'Unknown medium: poetry'):
           factory.create('poetry')







import unittest
from mock import sentinel
from tests.ditest import DependencyInjectionTestBase

class ExportingTests(DependencyInjectionTestBase):

    def setUp(self):
        super(ExportingTests, self).setUp()

    def export(self, *args, **kwargs):
        import niprov
        return niprov.export(*args, dependencies=self.dependencies, **kwargs)

    def test_Obtains_format_from_factory(self):
        self.export(form='json', medium='x')
        self.formatFactory.create.assert_any_call('json')
        self.export(form='xml', medium='x')
        self.formatFactory.create.assert_any_call('xml')

    def test_Obtains_medium_from_factory(self):
        self.export(medium='file', form='x')
        self.mediumFactory.create.assert_any_call('file')
        self.export(medium='stdout', form='x')
        self.mediumFactory.create.assert_any_call('stdout')

    def test_Passes_provenance_through_format_and_medium(self):
        self.format.serialize.return_value = 'serialized prov'
        self.medium.export.return_value = sentinel.mediumOutput
        out = self.export('a medium','a format')
        self.format.serialize.assert_any_call(self.repo.latest())
        self.medium.export.assert_called_with('serialized prov')
        self.assertEqual(sentinel.mediumOutput, out)

    def test_Without_specifics_returns_latest_files(self):
        out = self.export('a medium','a format')
        self.format.serialize.assert_called_with(self.repo.latest())

    def test_Can_export_one_file_specifically(self):
        out = self.export('a medium','a format',forFile='afile.f')
        self.format.serialize.assert_called_with(self.repo.byLocation('afile.f'))

    def test_Can_export_all_files_for_subject(self):
        out = self.export('a medium','a format',forSubject='Jane Doe')
        self.format.serialize.assert_called_with(
            self.repo.bySubject('Jane Doe'))

    def test_Passes_single_item_of_provenance_to_exporter(self):
        self.export('a medium','a format',forFile='xyz')
        self.format.serialize.assert_any_call(
            self.repo.byLocation('xyz'))

    def test_Can_export_stats(self):
        out = self.export('a medium','a format',statistics=True)
        self.format.serialize.assert_called_with(self.repo.statistics())

    def test_Completes_locationString_forFile(self):
        out = self.export('a medium','a format',forFile='afile.f')
        self.locationFactory.completeString.assert_any_call('afile.f')
        self.repo.byLocation.assert_called_with(self.locationFactory.completeString())

    def test_If_file_unknown_tells_listener(self):
        self.locationFactory.completeString.side_effect = lambda p: p
        self.repo.knowsByLocation.return_value = False
        self.repo.byLocation.side_effect = IndexError
        self.export('a medium','a format',forFile='xyz')
        self.listener.unknownFile.assert_called_with('xyz')

    def test_Can_export_pipeline_for_file(self):
        out = self.export('a medium','a format', forFile='a/b/c', pipeline=True)
        self.pipelineFactory.forFile.assert_called_with(self.repo.byLocation())
        self.format.serialize.assert_called_with(
            self.pipelineFactory.forFile())







import os


class Pipeline(object):

    def __init__(self, files):
        self.files = files
        self.roots = set([f for f in files if f.parents == []])
        def locationBranchRecurse(image):
            branch = {}
            loc = image.location.toString()
            children = [f for f in files if loc in f.parents]
            for child in children:
                branch[child.location.toString()] = locationBranchRecurse(child)
            return branch
        self.locationTree = {}
        for f in self.roots:
            self.locationTree[f.location.toString()] = locationBranchRecurse(f)

    def asFilenameTree(self):
        def toFilenameTree(d):
            new = {}
            for k, v in d.iteritems():
                if isinstance(v, dict):
                    v = toFilenameTree(v)
                newkey = os.path.basename(k.split(':')[1])
                new[newkey] = v
            return new
        return toFilenameTree(self.locationTree)







from niprov.dependencies import Dependencies


class BaseFile(object):

    def __init__(self, location, provenance=None, dependencies=Dependencies()):
        self.dependencies = dependencies
        self.listener = dependencies.getListener()
        self.filesystem = dependencies.getFilesystem()
        self.hasher = dependencies.getHasher()
        self.location = dependencies.getLocationFactory().fromString(location)
        self.formats = dependencies.getFormatFactory()
        if provenance:
            self.provenance = provenance
        else:
            self.provenance = {}
        self.provenance.update(self.location.toDictionary())
        self.path = self.provenance['path']

    def inspect(self):
        self.provenance['size'] = self.filesystem.getsize(self.path)
        self.provenance['created'] = self.filesystem.getctime(self.path)
        self.provenance['hash'] = self.hasher.digest(self.path)
        return self.provenance

    def attach(self, form='json'):
        """
        Not implemented for BaseFile parent class.

        Args:
            form (str): Data format in which to serialize provenance. Defaults 
                to 'json'.
        """
        pass

    def getProvenance(self, form):
        return self.formats.create(form).serialize(self)

    def getSeriesId(self):
        pass

    @property
    def parents(self):
        return self.provenance.get('parents', [])



#!/usr/bin/python
# -*- coding: UTF-8 -*-
from mock import Mock, sentinel, patch
from tests.ditest import DependencyInjectionTestBase


class StdoutMediumTests(DependencyInjectionTestBase):

    def test_Prints_formatted_provenance(self):
        from niprov.mediumstdout import StandardOutputMedium
        mprint = Mock()
        with patch('__builtin__.print') as mprint:
            exporter = StandardOutputMedium()
            out = exporter.export('Goodbye World!')
            mprint.assert_called_with('Goodbye World!')






#!/usr/bin/python
# -*- coding: UTF-8 -*-
from __future__ import print_function
from niprov.exceptions import UnknownFileError
from niprov.dependencies import Dependencies


class Commandline(object):

    vlevels = ['debug','info','warning','error']

    def __init__(self, dependencies=Dependencies()):
        self.config = dependencies.config
        self.verbosity = dependencies.config.verbosity
        assert self.verbosity in self.vlevels, "Unknown verbosity value"

    def fileFound(self, image):
        self.log('info', 'New file: {0}'.format(image.path))

    def fileFoundInSeries(self, img, series):
        template = 'Adding {0} file to series: {1}'
        nfiles = len(series.provenance['filesInSeries'])
        self.log('info', template.format(ordinal(nfiles), series.getSeriesId()))

    def missingDependencyForImage(self, lib, fpath):
        template = 'Missing python package "{0}" to read file: {1}'        
        self.log('warning', template.format(lib, fpath))

    def fileError(self, fpath):
        import traceback
        traceback.print_exc()
        self.log('warning', 'Error inspecting file: {0}'.format(fpath))

    def interpretedRecording(self, new, transform, parents):
        template = ('[provenance] Recorded the command [{1}] to create [{0}] '+
            'based on [{2}]')
        self.log('info', template.format(', '.join(new), transform, 
            ', '.join(parents)))

    def unknownFile(self, fpath):
        if self.config.dryrun:
            level = 'info'
        else:
            level = 'error'
        self.log(level, 'Unknown file: '+fpath, UnknownFileError)

    def knownFile(self, fpath):
        self.log('info', 'File already known: '+fpath)

    def renamedDicom(self, fpath):
        self.log('info', 'Renamed dicom file: '+fpath)

    def discoveryFinished(self, nnew, nadded, nfailed, ntotal):
        self.log('info', 'Discovered {0} new, added {1} to series, failed to read {2}, '
           'processed {3} total files.'.format(nnew, nadded, nfailed, ntotal))

    def mnefunEventReceived(self, operationName):
        self.log('info', 'Mnefun operation: '+operationName)

    def receivedBashCommand(self, command):
        self.log('info', 'Recording command: \n'+(' '.join(command)))

    def filesMarkedForApproval(self, images):
        paths = '\n'.join([img.path for img in images])
        self.log('info', 'Files marked for approval: \n{0}'.format(paths))

    def log(self, level, message, exceptionClass=None):
        if self.vlevels.index(level) >= self.vlevels.index(self.verbosity):
            if level == 'error':
                if exceptionClass is None:
                    raise NiprovError(message)
                else:
                    raise exceptionClass(message)
            else:
                print('[provenance:{0}] {1}'.format(level, message))
        

SUFFIXES = {1: 'st', 2: 'nd', 3: 'rd'}
def ordinal(num):
    if 10 <= num % 100 <= 20:
        suffix = 'th'
    else:
        # the second parameter is a default.
        suffix = SUFFIXES.get(num % 10, 'th')
    return str(num) + suffix






class FileMedium(object):

    def __init__(self, dependencies):
        self.filesys = dependencies.getFilesystem()
        self.clock = dependencies.getClock()

    def export(self, formattedProvenance):
        fname = 'provenance_{0}.txt'.format(self.clock.getNowString())
        self.filesys.write(fname, formattedProvenance)
        return fname



import unittest, os
from mock import Mock, patch, call
from tests.ditest import DependencyInjectionTestBase


class CommandlineTests(DependencyInjectionTestBase):

    def setUp(self):
        super(CommandlineTests, self).setUp()

    def test_raises_exception_for_unknown_file(self):
        from niprov.commandline import Commandline
        from niprov.exceptions import UnknownFileError
        cmd = Commandline()
        with self.assertRaisesRegexp(UnknownFileError, '.* /foo/bar.baz'):
            cmd.unknownFile('/foo/bar.baz')

    def test_Only_prints_if_appropriate_verbosity_setting(self):
        from niprov.commandline import Commandline
        self.dependencies.config.verbosity = 'warning'
        with patch('__builtin__.print') as mprint:
            cmd = Commandline(self.dependencies)
            cmd.log('info','Do you remember..')
            assert not mprint.called
            cmd.log('warning','Watch out!')
            mprint.assert_called_with('[provenance:warning] Watch out!')

    def test_If_on_dryrun_mode_an_unknown_file_is_not_an_error(self):
        from niprov.commandline import Commandline
        from niprov.exceptions import UnknownFileError
        self.dependencies.config.verbosity = 'warning' #default
        self.dependencies.config.dryrun = False
        cmd = Commandline(self.dependencies)
        cmd.log = Mock()
        cmd.unknownFile('abc')
        cmd.log.assert_called_with('error','Unknown file: abc', UnknownFileError)
        self.dependencies.config.dryrun = True
        cmd = Commandline(self.dependencies)
        cmd.log = Mock()
        cmd.unknownFile('abc')
        cmd.log.assert_called_with('info','Unknown file: abc', UnknownFileError)


        



from unittest import TestCase
from mock import Mock, sentinel, patch


class SimpleFormatTests(TestCase):

    def test_Statistics(self):
        from niprov.formatsimple import SimpleFormat
        exporter = SimpleFormat()
        out = exporter.serializeStatistics({'count':123,'totalsize':678})
        self.assertIn(' Number of files: 123', out)
        self.assertIn(' Total file size: 678', out)

    def test_Pipeline(self):
        from niprov.formatsimple import SimpleFormat
        pipeline = Mock()
        tree = {'raw.f':{'1a.f':{'2.f':{}},'1b.f':{}}}
        exp = ''
        exp += '+---raw.f\n'
        exp += '|   +---1a.f\n'
        exp += '|   |   +---2.f\n'
        exp += '|   +---1b.f\n'
        pipeline.asFilenameTree.return_value = tree
        exporter = SimpleFormat()
        out = exporter.serializePipeline(pipeline)
        self.assertEqual(exp, out)

    def test_SerializeSingle(self):
        from niprov.formatsimple import SimpleFormat
        exporter = SimpleFormat()
        out = exporter.serializeSingle(self.aFile())
        self.assertIn('a:                       b\n', out)

    def aFile(self):
        somefile = Mock()
        somefile.provenance = {'a':'b'}
        return somefile




from tests.ditest import DependencyInjectionTestBase
from mock import Mock


class DictFormatTests(DependencyInjectionTestBase):

    def setUp(self):
        super(DictFormatTests, self).setUp()

    def test_serialize_item_returns_its_prov(self):
        from niprov.formatdict import DictFormat
        form = DictFormat(self.dependencies)
        out = form.serializeSingle(self.aFile())
        self.assertEqual({'a':'b'}, out)

    def test_serialize_list_returns_its_provs(self):
        from niprov.formatdict import DictFormat
        form = DictFormat(self.dependencies)
        out = form.serializeList([self.aFile(), self.aFile(), self.aFile()])
        self.assertEqual([{'a':'b'},{'a':'b'},{'a':'b'}], out)

    def aFile(self):
        somefile = Mock()
        somefile.provenance = {'a':'b'}
        return somefile



from niprov.dependencies import Dependencies
from niprov.mediumstdout import StandardOutputMedium
from niprov.mediumdirect import DirectMedium
from niprov.mediumfile import FileMedium


class MediumFactory(object):

    def __init__(self, dependencies=Dependencies()):
        self.dependencies = dependencies

    def create(self, mediumName):
        if mediumName == 'stdout':
            return StandardOutputMedium()
        if mediumName == 'direct':
            return DirectMedium()
        if mediumName == 'file':
            return FileMedium(self.dependencies)
        raise ValueError('Unknown medium: '+str(mediumName))




from niprov.format import Format


class DictFormat(Format):

    def serializeSingle(self, item):
        return item.provenance

    def serializeList(self, fileList):
        return [item.provenance for item in fileList]





from tests.ditest import DependencyInjectionTestBase
from mock import Mock
import datetime


class XmlFormatTests(DependencyInjectionTestBase):

    def setUp(self):
        super(XmlFormatTests, self).setUp()

    def test_serialize_item_returns_string_with_xml_header(self):
        prolog = '<?xml version="1.0" encoding="UTF-8"?>'
        doc = '<prov:document'
        from niprov.formatxml import XmlFormat
        form = XmlFormat(self.dependencies)
        out = form.serializeSingle(self.aFile())
        self.assertIn(prolog, out)
        self.assertIn(doc, out)

    def test_serialize_list_creates_entity_for_each_file(self):
        from niprov.formatxml import XmlFormat
        form = XmlFormat(self.dependencies)
        out = form.serializeList([self.aFile(), self.aFile(), self.aFile()])
        self.assertEqual(3, out.count('<prov:entity'))

    def test_has_namespaces(self):
        from niprov.formatxml import XmlFormat
        form = XmlFormat(self.dependencies)
        out = form.serializeSingle(self.aFile())
        prov = 'http://www.w3.org/ns/prov#'
        self.assertNamespaceDefined('prov', prov, out)
        nfo = 'http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#'
        self.assertNamespaceDefined('nfo', nfo, out)
        dct = 'http://purl.org/dc/terms/'
        self.assertNamespaceDefined('dct', dct, out)

    def test_Entity_has_id(self):
        from niprov.formatxml import XmlFormat
        form = XmlFormat(self.dependencies)
        aFile = self.aFile()
        doc = self.parseDoc(form.serializeSingle(aFile))
        entity = doc.getElementsByTagName("prov:entity")[0]
        self.assertHasAttributeWithValue(entity, 'id', 'niprov:file0')

    def test_serialize_file_entity_has_fileUrl_prop(self):
        from niprov.formatxml import XmlFormat
        form = XmlFormat(self.dependencies)
        aFile = self.aFile()
        doc = self.parseDoc(form.serializeSingle(aFile))
        entity = doc.getElementsByTagName("prov:entity")[0]
        self.assertOneChildWithTagAndText(entity, 'nfo:fileUrl', 
            str(aFile.location.toUrl()))

    def test_serialize_file_entity_has_fileSize_prop(self):
        from niprov.formatxml import XmlFormat
        form = XmlFormat(self.dependencies)
        aFile = self.aFile()
        aFile.provenance['size'] = 56789
        doc = self.parseDoc(form.serializeSingle(aFile))
        entity = doc.getElementsByTagName("prov:entity")[0]
        self.assertOneChildWithTagAndText(entity, 'nfo:fileSize', str(56789))

    def test_serialize_file_entity_has_fileLastModified_prop(self):
        from niprov.formatxml import XmlFormat
        form = XmlFormat(self.dependencies)
        aFile = self.aFile()
        aFile.provenance['created'] = datetime.datetime.now()
        doc = self.parseDoc(form.serializeSingle(aFile))
        entity = doc.getElementsByTagName("prov:entity")[0]
        self.assertOneChildWithTagAndText(entity, 'nfo:fileLastModified', 
            aFile.provenance['created'].isoformat())

    def test_serialize_file_entity_has_hash(self):
        from niprov.formatxml import XmlFormat
        form = XmlFormat(self.dependencies)
        aFile = self.aFile()
        aFile.provenance['hash'] = 'abraca777'
        doc = self.parseDoc(form.serializeSingle(aFile))
        entity = doc.getElementsByTagName("prov:entity")[0]
        hasHash = self.assertOneChildWithTagName(entity, 'nfo:hasHash')
        hashref = self.getElementContent(hasHash)
        allHashes = doc.getElementsByTagName("nfo:FileHash")
        hashesWithId = [e for e in allHashes if e.attributes['id'].value==hashref]
        self.assertEqual(1, len(hashesWithId))
        hashEl = hashesWithId[0]
        self.assertOneChildWithTagAndText(hashEl, 'nfo:hashAlgorithm', 'MD5')
        self.assertOneChildWithTagAndText(hashEl, 'nfo:hashValue', 
            aFile.provenance['hash'])

    def test_FileHash_id_follows_sensible_format(self):
        from niprov.formatxml import XmlFormat
        form = XmlFormat(self.dependencies)
        aFile = self.aFile()
        aFile.provenance['hash'] = 'abraca777'
        doc = self.parseDoc(form.serializeSingle(aFile))
        entity = doc.getElementsByTagName("prov:entity")[0]
        fileId = entity.attributes['id'].value
        self.assertOneChildWithTagAndText(entity, 'nfo:hasHash', fileId+'.hash')

    def test_file_with_transformation_has_activity_w_corresponding_id(self):
        from niprov.formatxml import XmlFormat
        form = XmlFormat(self.dependencies)
        aFile = self.aFile()
        aFile.provenance['transformation'] = 'enchantment'
        doc = self.parseDoc(form.serializeSingle(aFile))
        ent, entId = self.assertOneChildWithTagThatHasAnId(doc, 'prov:entity')
        act, actId = self.assertOneChildWithTagThatHasAnId(doc, 'prov:activity')
        self.assertEqual(entId+'.xform', actId)

    def test_file_with_transformation_has_activity_w_label(self):
        from niprov.formatxml import XmlFormat
        form = XmlFormat(self.dependencies)
        aFile = self.aFile()
        aFile.provenance['transformation'] = 'enchantment'
        doc = self.parseDoc(form.serializeSingle(aFile))
        entity = doc.getElementsByTagName("prov:entity")[0]
        activity = doc.getElementsByTagName("prov:activity")[0]
        self.assertOneChildWithTagAndText(activity, 'dct:title', 'enchantment')

    def test_file_with_transformation_has_wasGeneratedBy_element(self):
        from niprov.formatxml import XmlFormat
        form = XmlFormat(self.dependencies)
        aFile = self.aFile()
        aFile.provenance['transformation'] = 'enchantment'
        doc = self.parseDoc(form.serializeSingle(aFile))
        ent, entId = self.assertOneChildWithTagThatHasAnId(doc, 'prov:entity')
        act, actId = self.assertOneChildWithTagThatHasAnId(doc, 'prov:activity')
        wasGen = self.assertOneChildWithTagName(doc, "prov:wasGeneratedBy")
        refEnt = self.assertOneChildWithTagName(wasGen, "prov:entity")
        refAct = self.assertOneChildWithTagName(wasGen, "prov:activity")
        self.assertHasAttributeWithValue(refEnt, 'prov:ref', entId)
        self.assertHasAttributeWithValue(refAct, 'prov:ref', actId)

    def test_file_with_transformation_has_wasGeneratedBy_with_time(self):
        from niprov.formatxml import XmlFormat
        form = XmlFormat(self.dependencies)
        aFile = self.aFile()
        aFile.provenance['transformation'] = 'enchantment'
        aFile.provenance['created'] = datetime.datetime.now()
        doc = self.parseDoc(form.serializeSingle(aFile))
        wasGen = self.assertOneChildWithTagName(doc, "prov:wasGeneratedBy")
        self.assertOneChildWithTagAndText(wasGen, 'prov:time', 
            aFile.provenance['created'].isoformat())

    def parseDoc(self, xmlString):
        from xml.dom.minidom import parseString
        dom = parseString(xmlString)
        return dom.documentElement

    def assertNamespaceDefined(self, prefix, ns, xmlString):
        doc = self.parseDoc(xmlString)
        self.assertHasAttributeWithValue(doc, 'xmlns:'+prefix, ns)
        
    def assertHasAttributeWithValue(self, element, attName, attValue):
        assert element.hasAttribute(attName), "Can't find attribute: "+attName
        self.assertEqual(attValue, element.attributes[attName].value)

    def assertOneChildWithTagThatHasAnId(self, parent, tag):
        elements = parent.getElementsByTagName(tag)
        elements = [e for e in elements if e.hasAttribute('id')]
        msg = 'Expected exactly one {0} with ID in {1}, but found {2}'
        nElem = len(elements)
        self.assertEqual(1, nElem, msg.format(tag, parent.tagName, nElem))
        return elements[0], elements[0].attributes['id'].value

    def assertOneChildWithTagName(self, parent, tag):
        elements = parent.getElementsByTagName(tag)
        msg = 'Expected exactly one {0} in {1}, but found {2}'
        nElem = len(elements)
        self.assertEqual(1, nElem, msg.format(tag, parent.tagName, nElem))
        return elements[0]

    def assertOneChildWithTagAndText(self, parent, tag, expValue):
        elem = self.assertOneChildWithTagName(parent, tag)
        self.assertEqual(expValue, self.getElementContent(elem))

    def getElementContent(self, element):
        rc = []
        for node in element.childNodes:
            if node.nodeType == node.TEXT_NODE:
                rc.append(node.data)
        return ''.join(rc)

    def aFile(self):
        somefile = Mock()
        somefile.provenance = {}
        somefile.location.toUrl.return_value = 'xkcd://HAL/location.loc'
        return somefile



from niprov.dependencies import Dependencies
from niprov.pipeline import Pipeline


class PipelineFactory(object):

    def __init__(self, dependencies=Dependencies()):
        self.files = dependencies.getRepository()

    def forFile(self, image):
        """Create a Pipeline object based on known files 'parents' field.
        """
        filesByLocation = {image.location.toString():image}

        def lookupRelativesRecursive(images, relationToLookFor):
            if relationToLookFor is 'parents':
                parentLocations = set()
                for image in images:
                    parentLocations.update(image.provenance.get('parents',[]))
                relatives = self.files.byLocations(list(parentLocations))
            elif relationToLookFor is 'children':
                thisGenerationLocations = [i.location.toString() for i in images]
                relatives = self.files.byParents(thisGenerationLocations)
            for relative in relatives:
                filesByLocation[relative.location.toString()] = relative
            if relatives:
                lookupRelativesRecursive(relatives, relationToLookFor)

        lookupRelativesRecursive([image], 'parents')
        lookupRelativesRecursive([image], 'children')
        return Pipeline(filesByLocation.values())



#!/usr/bin/python
# -*- coding: UTF-8 -*-
from niprov.dependencies import Dependencies


def export(medium, form, forFile=None, forSubject=None, 
        statistics=False, pipeline=False, dependencies=Dependencies()):
    """Publish or simply return provenance for selected files.

    To get provenance on one specific file, pass its path as the 'forFile' 
    argument. Alternatively, to get all files associated with a certain subject,
    use the 'forSubject' argument. If none of these is used, provenance for the 
    most recently registered files is reported.

    Args:
        medium (str): The medium in which to publish the provenance. 
            One of 'stdout' (print the provenance to the terminal), 'direct' 
            (return object to caller), or 'file' (write to a text file).
        form (str): The format in which to serialize the provenance. 
            One of 'json','xml','narrated','simple','dict'.
        forFile (str): Select one file based on this path.
        forSubject (str): Select files regarding this subject.
        statistics (bool): Print overall statistics.

    Returns:
        Depends on medium selected. 
    """
    formatFactory = dependencies.getFormatFactory()
    mediumFactory = dependencies.getMediumFactory()
    repository = dependencies.getRepository()
    listener = dependencies.getListener()
    location = dependencies.getLocationFactory()
    makePipeline = dependencies.getPipelineFactory()

    form = formatFactory.create(form)
    medium = mediumFactory.create(medium)
    
    if statistics:
        provenance = repository.statistics()
    elif forFile:
        forFile = location.completeString(forFile)
        if not repository.knowsByLocation(forFile):
            listener.unknownFile(forFile)
            return
        provenance = repository.byLocation(forFile)
        if pipeline:
            provenance = makePipeline.forFile(provenance)
    elif forSubject:
        provenance = repository.bySubject(forSubject)
    else:
        provenance = repository.latest()

    formattedProvenance = form.serialize(provenance)
    return medium.export(formattedProvenance)


def get(forFile=None, forSubject=None, statistics=False, pipeline=False, 
    dependencies=Dependencies()):
    """Shortcut for export(medium='direct', form='object').
    """
    return export(medium='direct', form='object', 
        forFile=forFile, forSubject=forSubject, statistics=statistics, 
        pipeline=pipeline, dependencies=dependencies)

def print_(forFile=None, forSubject=None, statistics=False, pipeline=False, 
    dependencies=Dependencies()):
    """Shortcut for export(medium='stdout', form='simple').
    """
    return export(medium='stdout', form='simple', 
        forFile=forFile, forSubject=forSubject, statistics=statistics, 
        pipeline=pipeline, dependencies=dependencies)

def backup(dependencies=Dependencies()):
    """Shortcut for export(medium='file', form='json') for all provenance.
    """
    return export(medium='file', form='json', dependencies=dependencies)




import os
from mock import Mock, patch, call
from tests.ditest import DependencyInjectionTestBase


class PipelineTests(DependencyInjectionTestBase):

    def setUp(self):
        super(PipelineTests, self).setUp()

    def test_asFilenameTree(self):
        from niprov.pipeline import Pipeline
        a = self.fileWithLocationAndParents('a:/p/a.f',[])
        b = self.fileWithLocationAndParents('a:/p/c.f',['a:/p/a.f'])
        c = self.fileWithLocationAndParents('b:/c/d/d.f',['a:/p/a.f'])
        d = self.fileWithLocationAndParents('b:/c/d/e.f',['a:/p/c.f'])
        pipelineFiles = [a,b,c,d]
        pipeline = Pipeline(pipelineFiles)
        tree = pipeline.asFilenameTree()
        self.assertEqual({'a.f':{'c.f':{'e.f':{}},'d.f':{}}}, tree)

    def test_Determines_roots_on_creation(self):
        from niprov.pipeline import Pipeline
        b = self.fileWithLocationAndParents('b',['r1','a'])
        r1 = self.fileWithLocationAndParents('r1',[])
        a = self.fileWithLocationAndParents('a',['r2'])
        r2 = self.fileWithLocationAndParents('r2',[])
        pipelineFiles = [a,b,r1,r2]
        pipeline = Pipeline(pipelineFiles)
        self.assertEqual(set([r1,r2]), pipeline.roots)

    def fileWithLocationAndParents(self, loc, parents):
        f = Mock()
        f.location.toString.return_value = loc
        f.provenance = {'parents':parents}
        f.parents = parents
        return f






#!/usr/bin/python
# -*- coding: UTF-8 -*-
from mock import Mock, sentinel, patch
from tests.ditest import DependencyInjectionTestBase


class FileMediumTests(DependencyInjectionTestBase):

    def test_Writes_formattedProvenance_to_file(self):
        from niprov.mediumfile import FileMedium
        medium = FileMedium(self.dependencies)
        out = medium.export('Once upon a time..')
        self.assertEqual('Once upon a time..', 
            self.filesys.write.call_args[0][1])

    def test_Comes_up_with_filename(self):
        from niprov.mediumfile import FileMedium
        self.clock.getNowString.return_value = 'hammertime'
        medium = FileMedium(self.dependencies)
        out = medium.export('provstr')
        self.filesys.write.assert_called_with('provenance_hammertime.txt',
            'provstr')

    def test_Returns_filename(self):
        from niprov.mediumfile import FileMedium
        self.clock.getNowString.return_value = 'hammertime'
        medium = FileMedium(self.dependencies)
        out = medium.export('provstr')
        self.assertEqual('provenance_hammertime.txt', out)







from mock import Mock
from tests.ditest import DependencyInjectionTestBase
import datetime, random


class JsonFileTest(DependencyInjectionTestBase):

    def setUp(self):
        super(JsonFileTest, self).setUp()
        self.dependencies.getConfiguration().database_url = ''

    def imageWithProvenance(self, prov):
        img = Mock()
        img.provenance = prov
        if 'location' in prov:
            img.location.toString.return_value = prov['location']
        return img

    def test_Add(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        img1 = self.imageWithProvenance({'location':'1','foo':'baz'})
        repo.all = Mock()
        repo.all.return_value = [img1]
        image = self.imageWithProvenance({'location': '2','foo':'bar'})
        repo.add(image)
        self.serializer.serializeList.assert_called_with(
            [img1, image])
        self.filesys.write.assert_called_with(repo.datafile, 
            self.serializer.serializeList())

    def test_Update(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        img1 = self.imageWithProvenance({'location':'1','path':'a'})
        img2 = self.imageWithProvenance({'location':'2','path':'b'})
        repo.all = Mock()
        repo.all.return_value = [img1, img2]
        image = self.imageWithProvenance({'location': '2','foo':'bar'})
        repo.update(image)
        self.serializer.serializeList.assert_called_with(
            [img1,image])
        self.filesys.write.assert_called_with(repo.datafile, 
            self.serializer.serializeList())

    def test_knowsByLocation(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        repo.byLocation = Mock()
        self.assertTrue(repo.knowsByLocation(''))
        repo.byLocation.side_effect = IndexError
        self.assertFalse(repo.knowsByLocation(''))

    def test_byLocation(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        img1 = self.imageWithProvenance({'location':'1','path':'a'})
        img2 = self.imageWithProvenance({'location':'2','path':'b'})
        repo.all = Mock()
        repo.all.return_value = [img1, img2]
        out = repo.byLocation('2')
        self.assertEqual(img2, out)

    def test_byLocation_works_for_file_in_series(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        img1 = self.imageWithProvenance({'location':'1','path':'a'})
        img3 = self.imageWithProvenance({'location':'3','filesInSeries':['boo','bah']})
        repo.all = Mock()
        repo.all.return_value = [img1, img3]
        out = repo.byLocation('boo')
        self.assertEqual(img3, out)

    def test_updateApproval(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        img = Mock()
        img.provenance = {'fiz':'baf','approval':'notsure'}
        def assertion(img):
            self.assertEqual({'fiz':'baf','approval':'excellent!'}, img.provenance)
        repo.byLocation = Mock()
        repo.byLocation.return_value = img
        repo.update = Mock()
        repo.update.side_effect = assertion
        repo.updateApproval('/p/f1','excellent!')

    def test_bySubject(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        img1 = self.imageWithProvenance({'subject':'john','a':'b'})
        img2 = self.imageWithProvenance({'subject':'tim','a':'d'})
        img3 = self.imageWithProvenance({'subject':'john','a':'f'})
        repo.all = Mock()
        repo.all.return_value = [img1, img2, img3]
        out = repo.bySubject('john')
        self.assertEqual([img1, img3], out)

    def test_byApproval(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        img1 = self.imageWithProvenance({'approval':'y','a':'b'})
        img2 = self.imageWithProvenance({'approval':'x','a':'d'})
        img3 = self.imageWithProvenance({'approval':'x','a':'f'})
        repo.all = Mock()
        repo.all.return_value = [img1, img2, img3]
        out = repo.byApproval('x')
        self.assertEqual([img2, img3], out)

    def test_latest(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        img1 = self.imageWithProvenance({'added':datetime.datetime(1982, 1, 5)})
        img2 = self.imageWithProvenance({'added':datetime.datetime(1982, 2, 5)})
        img3 = self.imageWithProvenance({'added':datetime.datetime(1982, 3, 5)})
        img4 = self.imageWithProvenance({'added':datetime.datetime(1982, 4, 5)})
        img5 = self.imageWithProvenance({'added':datetime.datetime(1982, 5, 5)})
        repo.all = Mock()
        repo.all.return_value = [img1,img2,img3,img4,img5]
        out = repo.latest(3)
        self.assertEqual([img5, img4, img3], out)

    def test_stats(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        repo.all = Mock()
        records = [{},{},{},{},{},{},{},{},{},{},{}]
        totalsize = 0
        for r in records:
            r['size'] = random.randint(1,1000)
            totalsize += r['size']
        repo.all.return_value = [self.imageWithProvenance(r) for r in records]
        out = repo.statistics()
        self.assertEqual(11, out['count'])
        self.assertEqual(totalsize, out['totalsize'])

    def test_stats_transient_file(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        repo.all = Mock()
        records = [{},{},{},{},{},{},{},{},{},{},{}]
        totalsize = 0
        for r in records:
            r['size'] = random.randint(1,1000)
            totalsize += r['size']
        totalsize -= records[3]['size']
        del records[3]['size']
        repo.all.return_value = [self.imageWithProvenance(r) for r in records]
        out = repo.statistics()
        self.assertEqual(totalsize, out['totalsize'])
        
    def test_byId(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        img1 = self.imageWithProvenance({'id':'1'})
        img2 = self.imageWithProvenance({'id':'2'})
        img3 = self.imageWithProvenance({'id':'3'})
        repo.all = Mock()
        repo.all.return_value = [img1, img2, img3]
        out = repo.byId('2')
        self.assertEqual(img2, out)

    def test_bySubject_doesnt_balk_if_no_subject_field(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        img1 = self.imageWithProvenance({'a':'b'})
        img2 = self.imageWithProvenance({'subject':'tim','a':'d'})
        img3 = self.imageWithProvenance({'subject':'john','a':'f'})
        repo.all = Mock()
        repo.all.return_value = [img1, img2, img3]
        out = repo.bySubject('john')

    def test_byLocations(self):
        self.fileFactory.fromProvenance.side_effect = lambda p: 'img_'+p['a']
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        img1 = self.imageWithProvenance({'location':'i'})
        img2 = self.imageWithProvenance({'location':'j'})
        img3 = self.imageWithProvenance({'location':'m'})
        img4 = self.imageWithProvenance({'location':'f'})
        img5 = self.imageWithProvenance({'location':'x'})
        repo.all = Mock()
        repo.all.return_value = [img1,img2,img3,img4,img5]
        out = repo.byLocations(['j','f','k'])
        self.assertEqual([img2, img4], out)

    def test_byParents(self):
        self.fileFactory.fromProvenance.side_effect = lambda p: 'img_'+p['l']
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        img1 = self.imageWithProvenance({'parents':[],'l':'a'})
        img2 = self.imageWithProvenance({'parents':['x','a'],'l':'b'})
        img3 = self.imageWithProvenance({'parents':['b'],'l':'b'})
        img4 = self.imageWithProvenance({'parents':['c','y'],'l':'d'})
        img5 = self.imageWithProvenance({'parents':['d'],'l':'e'})
        repo.all = Mock()
        repo.all.return_value = [img1, img2, img3, img4, img5]
        out = repo.byParents(['x','y'])
        self.assertEqual([img2, img4], out)




from mock import Mock, patch, sentinel
from tests.ditest import DependencyInjectionTestBase
from datetime import datetime


class ExportingTest(DependencyInjectionTestBase):

    def setUp(self):
        super(ExportingTest, self).setUp()
        self.tempRepo = Mock()

    def test_import(self):
        import niprov.importing 
        self.tempRepo.all.return_value = [sentinel.p1, sentinel.p2]
        niprov.importing.JsonFile = self.patchJsonFileConstructor()
        niprov.importing.importp('target_file', dependencies=self.dependencies)
        assert self.JsonFileCtr.called, "Did not create a JsonFile repo."
        urlUsed = self.tempRepo.dependencies.getConfiguration().database_url
        self.assertEqual(urlUsed, 'target_file')
        self.repo.add.assert_any_call(sentinel.p1)
        self.repo.add.assert_any_call(sentinel.p2)

    def patchJsonFileConstructor(self):
        self.JsonFileCtr = Mock()
        def ctr(dependencies): 
            self.tempRepo.dependencies = dependencies
            return self.tempRepo
        self.JsonFileCtr.side_effect = ctr
        return self.JsonFileCtr






from niprov.dependencies import Dependencies
from niprov.jsonfile import JsonFile
from datetime import datetime as dt

def importp(filepath, dependencies=Dependencies()):
    """Add provenance in bulk from a file, such as saved by export().

    Named importp as opposed to import because the latter is a reserved word
    in Python.

    This can serve as a backup, migration tool, or for exchange.
    """
    repository = dependencies.getRepository()
    importDeps = Dependencies()
    importDeps.getConfiguration().database_url = filepath
    importRepo = JsonFile(importDeps)
    allFiles = importRepo.all()
    for pfile in allFiles:
        repository.add(pfile)



import unittest
import os, shutil


class ContextApiTests(unittest.TestCase):

    def setUp(self):
        self.dbpath = os.path.expanduser(os.path.join('~','provenance_test.json'))
        if os.path.exists(self.dbpath):
            os.remove(self.dbpath)
        os.mkdir('temp')
        from niprov import Context
        self.provenance = Context()
        self.provenance.config.database_type = 'file'
        self.provenance.config.database_url = self.dbpath

    def tearDown(self):
        shutil.rmtree('temp')

    def touch(self, path):
        with open(path,'w') as tempfile:
            tempfile.write('0')

    def test_Log(self):
        self.provenance.discover('testdata')
        newfile = 'temp/smoothed.test'
        self.touch(newfile)
        parent = os.path.abspath('testdata/eeg/stub.cnt')
        self.provenance.log(newfile, 'test', parent)
        testfpath = os.path.abspath(newfile)
        img = self.provenance.get(forFile=testfpath)
        self.assertEqual(img.provenance['subject'], 'Jane Doe')
        self.assertEqual(img.provenance['size'], os.path.getsize(newfile))

    def test_Export_Import(self):
        from niprov.exceptions import UnknownFileError
        self.provenance.discover('testdata')
        discoveredFile = os.path.abspath('testdata/eeg/stub.cnt')
        self.assertIsNotNone(self.provenance.get(forFile=discoveredFile))
        backupFilepath = self.provenance.backup()
        os.remove(self.dbpath) # get rid of existing data.
        with self.assertRaises(UnknownFileError):
            self.provenance.get(forFile=discoveredFile)
        self.provenance.importp(backupFilepath)
        self.assertIsNotNone(self.provenance.get(forFile=discoveredFile))

    def test_Attach_provenance_string_in_file_based_on_config(self):
        import nibabel
        self.provenance.config.attach = True
        newfile = os.path.abspath('temp/fileX.nii.gz')
        shutil.copy('testdata/nifti/fieldmap.nii.gz', newfile)
        self.provenance.add(newfile)
        img = nibabel.load(newfile)
        self.assertEqual(img.get_header().extensions.count('comment'), 1)
        self.assertEqual(img.get_header().extensions[0].get_code(), 6)
        content = img.get_header().extensions[0].get_content()
        self.assertIn('"path": "{0}"'.format(newfile), content)
        
        

if __name__ == '__main__':
    unittest.main()

